#include "grafo.h"

arista::arista(int d, float c, arista *s){
  destino = d; 
  coste   = c;
  sig     = s; 
}

Grafo::Grafo(int nvert){
  numVert = nvert;
  vec = new arista*[numVert];
  color = new GrafoColor[numVert];
  for (int i=0; i<numVert; ++i){
    vec[i] = 0;
    color[i] = WHITE;
  }
}

Grafo::~Grafo(){
  for (int i=0; i<numVert; ++i)
    while (vec[i] != 0) {
      arista *aux = vec[i];
      vec[i] = vec[i]->sig;
      delete aux;
    }
  delete[] vec;
  delete[] color;
}

void Grafo::addEdge(int from, int to, float coste){
  if (!hasEdge(from,to)) {
    vec[from] = new arista(to, coste, vec[from]);
  }
}

bool Grafo::hasEdge(int from, int to){
  for (arista *aux=vec[from]; aux; aux=aux->sig) {
    if (aux->destino == to) return true;
  }
  return false;
}

float Grafo::dijkstra(int origen, bool pre, int rep){
  const float infinito = 1e99;
  float *D = new float[numVert];
  bool  *S = new  bool[numVert];
  int *Pred = new int[numVert];
  for (int i=0;i<numVert;++i){
    D[i]    = infinito;
    S[i]    = false; // no seleccionado
    Pred[i] = -1;
  }
  D[origen] = 0;
  for (int iteraciones = 0; iteraciones<numVert; iteraciones++) {
    // buscar menor valor D de un vertice NO seleccionado
    float menor = infinito;
    int quien_menor = -1;
    for (int i=0;i<numVert;++i) 
      if (!S[i] && D[i]<menor) {
	menor = D[i];
	quien_menor = i;
      }
    // ya conocemos el menor
    if (quien_menor == -1) break;
    S[quien_menor] = true;
    for (arista *p=vec[quien_menor]; p!=0; p=p->sig) {
      if (!S[p->destino] && D[quien_menor]+p->coste < D[p->destino]){
	D[p->destino]    = D[quien_menor]+p->coste;
        Pred[p->destino] = quien_menor;
      }
    }
  }
  
  float max=-1;
  int maxi =-1;
  for(int i=0;i<numVert;i++)
    if(D[i] > max && D[i]!=infinito){
      max = D[i];
      maxi = i;
    }
  //if(!pre){
  //std::cout <<"camino (al reves): " << rep << " -> " << max << std::endl;
  //for (int i=maxi; i!=-1; i=Pred[i]) std::cout << "i: " << i << ", max: " << max << " \n";
  for(int i=maxi;i!=-1;i=Pred[i]) color[i]=BLACK;
  //}
  delete[] D;
  delete[] S;
  return max;
}

arista **Grafo::get_vec(){
  return vec;
}
